/* ---------------------------------------------------------------------
Program to hex dump SEGY disk files
   Will display:  EBCDIC header values
		  binary header values
		  trace header values
		  sample hex values
		  actual segy sample values
---------------------------------------------------------------------- */

#include <stdio.h>

#ifndef DUMPSEGY_H
#define DUMPSEGY_H 1
#include "dumpsegy.h"
#endif


int main(int argc, char *argv[])
{
  EBCDIC     ebcdicHdr;
  BINHEADER  binaryHdr;
  TRCHEADER  traceHdr;
  TRCSAMPLES samples;
  FILE_INFO  fileInfo;
  CMDLINE    cmdLine;

  unsigned char ebcdicBuffer[3200],binaryBuffer[400],traceHdrBuffer[240],
		sampleBuffer[4];
  int           iStatus,i;
  char          ch,*ptr;
  long          lowByte,highByte,byteCount,traceNumber;

  cmdLine.e=0;
  cmdLine.b=0;
  cmdLine.t=0;
  cmdLine.v=0;
  cmdLine.n=0;
  cmdLine.c=0;
  cmdLine.r=0;
  cmdLine.firstTrace=0L;
  cmdLine.lastTrace=0L;
  cmdLine.increment=1L;
  cmdLine.inputFile=NULL;
  cmdLine.outputFile=stdout;
  cmdLine.outputFileName[0]='\0';

  if(argc<2)
    {
      Usage();
      exit(1);
    }

  if(argc>=3)
    {
      GetCmdLine(&cmdLine,argc,argv);
    }

    if(strlen(cmdLine.outputFileName)>0)
      {
	if(cmdLine.c)
	  cmdLine.outputFile=fopen(cmdLine.outputFileName,"wb");
	else
	  cmdLine.outputFile=fopen(cmdLine.outputFileName,"w");

	if(cmdLine.outputFile==NULL)
	  {
	    fprintf(stderr,"\nERROR OPENING %s...OUTPUT WILL GO TO stdout\n",
		    cmdLine.outputFileName);
	    Continue();
	    cmdLine.outputFile=stdout;
	  }
      }

  /*--- OPEN INPUT FILE ---*/
  if((cmdLine.inputFile=fopen(argv[1],"rb"))==NULL)
  {
    Usage();
    exit(1);
  }
  strcpy(fileInfo.fileName,argv[1]);

  /*--- GET EBCDIC HEADER ---*/
  if(cmdLine.e==TRUE)
    {
      GoToByte(cmdLine.inputFile,1L);
      if((fread(ebcdicBuffer,EBCDICHEADER,1,cmdLine.inputFile))==NULL)
	{
	  fprintf(stderr,"ERROR reading EBCDIC header\n\n");
	  exit(1);
	}
    }


  /*--- GET BINARY HEADER ---*/
  GoToByte(cmdLine.inputFile,3201L);
  if((fread(binaryBuffer,BINARYHEADER,1,cmdLine.inputFile))==NULL)
    {
      fprintf(stderr,"ERROR reading BINARY header\n\n");
      exit(1);
    }

  iStatus=GetBinaryHeader(binaryBuffer,&binaryHdr);

  fileInfo.numberOfSamples=binaryHdr.samplesPerTraceReel;
  fileInfo.sampleRate=(float)binaryHdr.sampleRateReel;
  fileInfo.bytesPerTrace=(binaryHdr.samplesPerTraceReel*4)+240;
  fileInfo.maxTime=binaryHdr.sampleRateReel/1000*binaryHdr.samplesPerTraceReel;
  fseek(cmdLine.inputFile,0L,SEEK_END);
  fileInfo.bytesInFile=ftell(cmdLine.inputFile);
  rewind(cmdLine.inputFile);
  fileInfo.tracesInFile=(int)((fileInfo.bytesInFile-3600)/fileInfo.bytesPerTrace);

  if(!cmdLine.c)
    {
      if(cmdLine.outputFile==stdout)
	for(i=0;i<26;i++)
	   fprintf(cmdLine.outputFile,"\n");
      if(cmdLine.outputFile==stdout)
	{
	  fprintf(cmdLine.outputFile,"===========================================================\n");
	  fprintf(cmdLine.outputFile,"                       DUMPSEGY V%0.2f\n",VERSION);
	  fprintf(cmdLine.outputFile,"       U.S. Geological Survey Open File Report 92-590\n");
	  fprintf(cmdLine.outputFile,"                       F.N. Zihlman\n");
	  fprintf(cmdLine.outputFile,"             P.O. Box 25046, Mail Stop 960\n");
	  fprintf(cmdLine.outputFile,"                     Denver, CO 80225\n");
	  fprintf(cmdLine.outputFile,"                        303-236-5741\n");
	  fprintf(cmdLine.outputFile,"=========================================================\n\n");
	}

      PrintFileInfo(&cmdLine,&fileInfo);
      if(cmdLine.outputFile==stdout)
	{
	  Continue();
	}
    }

  /*--- PROCESS EBCDIC HEADER ---*/
  if(cmdLine.e==TRUE)
    {
      if(cmdLine.c)
	fwrite(ebcdicBuffer,EBCDICHEADER,1,cmdLine.outputFile);
      else
	{
	  iStatus=GetEBCDICHeader(ebcdicBuffer, &ebcdicHdr);

	  if(iStatus==TRUE)
	    {
	      PrintEBCDICHeader(cmdLine.outputFile,&ebcdicHdr);
	    }
	}
    }

  /*----- PROCESS BINARY HEADER -----*/
  if(cmdLine.b==TRUE)
    {
      if(cmdLine.c)
	fwrite(binaryBuffer,BINARYHEADER,1,cmdLine.outputFile);
      else
	PrintBinaryHeader(cmdLine.outputFile,&binaryHdr);
    }


  if((cmdLine.n)&&(!cmdLine.t))
    exit(0);

  if(cmdLine.firstTrace!=0L)
    traceNumber=cmdLine.firstTrace;
  else
    traceNumber=1L;

  while(traceNumber!=0L)
  {
    if(!cmdLine.r)
      {
	printf("\n\ntrace number (enter 0 to quit): ");
	scanf("%ld",&traceNumber);
	fflush(stdin);
      }

    if((traceNumber>=1L)&&(traceNumber<=fileInfo.tracesInFile))
      {
	iStatus=GoToTrace(cmdLine.inputFile,traceNumber,fileInfo.bytesPerTrace);
	lowByte=3600L+(traceNumber-1L)*fileInfo.bytesPerTrace+240+1;
	highByte=lowByte+15L;
      }
    else
      iStatus=0;

    /*--- GET DATA TRACE HEADER ---*/
    if(iStatus==1)
      {
	if((fread(traceHdrBuffer,TRACEHEADER,1,cmdLine.inputFile))==NULL)
	  {
	    fprintf(stderr,"ERROR reading TRACE header\n\n");
	    exit(1);
	  }

	if(cmdLine.c)
	  fwrite(traceHdrBuffer,TRACEHEADER,1,cmdLine.outputFile);
	else
	  {
	    iStatus=GetTraceHeader(traceHdrBuffer,&traceHdr);

	    if((iStatus==TRUE)&&(cmdLine.t==TRUE))
	      {
		PrintTraceHeader(cmdLine.outputFile,&traceHdr,fileInfo.bytesPerTrace,traceNumber);
	      }
	  }

	if((iStatus==TRUE)&&(!cmdLine.n))
	  {
	    if(cmdLine.c)
	      printf("processing trace [%ld]\n",traceNumber);

	    for(i=0;i<fileInfo.numberOfSamples;i++)
	       {
		 if((fread(sampleBuffer,4,1,cmdLine.inputFile))==NULL)
		   {
		     printf("NULL returned reading trace sample\n");
		     break;
		   }

		 if(cmdLine.c)
		   {
		     fwrite(sampleBuffer,4,1,cmdLine.outputFile);
		   }
		 else
		   {
		     if((i==0)||((i%4)==0))
		       {
			 fprintf(cmdLine.outputFile,"\n");
			 fprintf(cmdLine.outputFile,"[%5ld - %5ld] ",lowByte,highByte);
			 lowByte=highByte+1L;
			 highByte+=16L;
		       }

		     PrintSamples(&cmdLine,sampleBuffer);
		   }
	       }

	    if(!cmdLine.c)
	      fprintf(cmdLine.outputFile,"\n\n");
	  }
      }

      if(cmdLine.r)
	{
	  if(cmdLine.firstTrace<=cmdLine.lastTrace)
	    {
	      if(traceNumber+cmdLine.increment<=cmdLine.lastTrace)
		traceNumber+=cmdLine.increment;
	      else
		traceNumber=0L;
	    }
	  else if(traceNumber>=cmdLine.lastTrace-cmdLine.increment)
	    traceNumber+=cmdLine.increment;
	  else
	    traceNumber=0L;
	}

  }
  fclose(cmdLine.inputFile);
  if(cmdLine.outputFile!=stdout)
    fclose(cmdLine.outputFile);

  return 0;
}

